#!/usr/bin/perl 
#------------------------------------------------------------------------------
# Name:		pdadmin.cgi
#
# Author:	Peter Sundstrom (Ginini Technologies Limited)
#
# Source:	http://postcard-direct.com/
#
# Version:	2.0.1
#
# Copyright:	(c)1999-2005 Peter Sundstrom.
#		All rights reserved.
#
#------------------------------------------------------------------------------

use strict;

#
# Required modules
#
use File::Basename;
use File::Copy;

use CGI;

BEGIN {
	require CGI::Carp;

	sub handle_errors {
		my $msg=shift;
		print qq(<html><head><title>Perl Error</title></head><body>\n);
		print qq(<font face="Verdana,Arial,Helvetica">);
		print qq(<h1>Perl Error</h1>\n);
		print qq(<h3>Error Message(s)</h3><pre>$msg</pre>\n);
		print qq(<br>Check <a href="http://postcard-direct.com/perlerrors.html">Perl Errors</a> for a list of common Perl errors and how to fix them.\n);
		print qq(<br>Also check the <a href="http://postcard-direct.com/faq.html">Postcard Direct FAQ</a>.\n);
		print qq(</body></html>\n);
	}

	#
	# Setting custom messages is only available in
	# CGI::CARP Version 1.09 and above.
	#
	if ($CGI::Carp::VERSION < 1.09) {
		CGI::Carp->import('carpout','fatalsToBrowser');
	}
	else {
		CGI::Carp->import('carpout','fatalsToBrowser','set_message');
        	set_message(\&handle_errors);
	}


	use FindBin qw($Bin);

	if ($Bin) {
		if ($Bin eq '/') {
			$Bin='';
		}
		else {
			$Bin .= '/';
			chdir $Bin or die "Can not change directory to $Bin $!\n";
		}
	}

	#
	# Determine website name and document root
	#
	use vars qw($document_root);

	if ($ENV{DOCUMENT_ROOT}) {
        	$document_root=$ENV{DOCUMENT_ROOT};
	}
	elsif ($ENV{PATH_TRANSLATED}) {
        	($document_root = $ENV{PATH_TRANSLATED}) =~ s/$ENV{PATH_INFO}//g;
}

	my $siteurl=$ENV{SERVER_NAME};

	require "${Bin}configs/pdconfig.txt";
	use vars qw($modules);
	$modules="${Bin}modules";

}

#
# Use the PD common functions and global config file
#
use lib $modules;

#
# For Cookie encryption
#
require tea;
Crypt::Tea->import();

#
# Set a key for TEA encryption
#
use vars qw($tkey);
$tkey = $ENV{'SERVER_NAME'};

# 
# Common routines
#
use pdcommon;

#
# Common variables used throughout the script
#
use vars qw($version $expire);
$version='2.0.1';

#
# Declare globals from the configuration files
#
use vars qw(	$pdurl
		$maxpost
		$pdroot
		$document_root
		$images
		$lang 
		$lang_name
		$badwords_list
		$object_list
		$midi_list
		$design_list
		$admin_dir
		$cache_dir
		$config_dir
		$template_dir
		$design_dir
		$postcard_dir
		$style_dir
		$log_dir
		$upload_dir
		$log_file
		$stylesheet
);
		
$CGI::POST_MAX  = $maxpost;	# Restrict POST size
$CGI::DISABLE_UPLOADS = 1;	# Disable uploads

my $q = new CGI;
my $script = $q->url(-absolute=>1);
my $action=$q->param('action');
my $design=$q->param('design');
my $template=$q->param('template');
my $configuration=$q->param('configuration');
my $language=$q->param('language');

#
# Extract language code and description
#
if ($q->param('language')) {
	($lang,$lang_name) = $q->param('language') =~ /(\w+)\|(\w+)/;
}
else {
	$language='unknown';
}



#
# Set appropriate configuration file
#
$configuration='pdconfig.txt' unless $configuration;
my $configfile="$config_dir/$configuration";

#
# Use the configuration file
#
require $configfile;

#
# Set the default stylesheet
#
if($q->param('stylesheet')) {
	$stylesheet=$q->param('stylesheet');
}
else {
	$stylesheet=$stylesheet;
}

#
# If no action has been specified, then present login page
#
Login() if ($action eq '' or $action eq 'Login');

#
# If this script has been renamed with a .pl extention 
# ensure pdedit is called correctly.
#
my $pdedit;
my $pdview;

if ($script =~ /\.pl/) {
	$pdedit='pdedit.pl';
	$pdview='pdview.pl';
}
else {
	$pdedit='pdedit.cgi';
	$pdview='pdview.cgi';
}


#
# Make an attempt to ensure this script has been invoked 
# on this server, or least from a specified server.
#
# Currently commented due to not all browsers
# generate referer information.  It can be uncommented
# if you desire.
#
#CheckReferer() unless ($action =~ /Usage|Preview/);

SetPassword() if ($action eq 'SetPassword');

#
# Check user authorisation (unless logging in).
#
CheckAuth() if ($action ne 'CheckLogin' and $action ne 'ChangePassword' and $action ne 'Logout');


#
# Check what action to perform.
#
if ($action eq 'CheckLogin') {
	CheckLogin();
}
elsif ($action eq 'MainMenu') {
	MainMenu();
}
elsif ($action eq 'Select Language') {
	SelectLanguage();
}
elsif ($action eq 'Select Configuration') {
	Operation('Configuration','Select',$config_dir,'txt',$configuration);
}
elsif ($action eq 'Copy Configuration') {
	Operation('Configuration','Copy',$config_dir,'txt',$configuration);
}
elsif ($action eq 'CopyConfiguration') {
	CopyFile('Configuration',$config_dir);
}
elsif ($action eq 'Delete Configuration') {
	Operation('Configuration','Delete',$config_dir,'txt',$configuration);
}
elsif ($action eq 'DeleteConfiguration') {
	DeleteFile('Configuration',$config_dir);
}
elsif ($action eq 'Select Template') {
	Operation('Template','Select',$template_dir,'txt',$template);
}
elsif ($action eq 'Copy Template') {
	Operation('Template','Copy',$template_dir,'txt',$template);
}
elsif ($action eq 'CopyTemplate') {
	CopyFile('Template',$template_dir);
}
elsif ($action eq 'Delete Template') {
	Operation('Template','Delete',$template_dir,'txt',$template);
}
elsif ($action eq 'DeleteTemplate') {
	DeleteFile('Template',$template_dir);
}
elsif ($action eq 'Select Stylesheet') {
	Operation('Stylesheet','Select',$style_dir,'css',$stylesheet);
}
elsif ($action eq 'Copy Stylesheet') {
	Operation('Stylesheet','Copy',$style_dir,'css',$stylesheet);
}
elsif ($action eq 'CopyStylesheet') {
	CopyFile('Stylesheet',$style_dir);
}
elsif ($action eq 'Delete Stylesheet') {
	Operation('Stylesheet','Delete',$style_dir,'css',$stylesheet);
}
elsif ($action eq 'DeleteStylesheet') {
	DeleteFile('Stylesheet',$style_dir);
}
elsif ($action eq 'Select Design') {
	Operation('Design','Select',$design_dir,'txt',$design);
}
elsif ($action eq 'Copy Design') {
	Operation('Design','Copy',$design_dir,'txt',$design);
}
elsif ($action eq 'CopyDesign') {
	CopyFile('Design',$design_dir);
}
elsif ($action eq 'Delete Design') {
	Operation('Design','Delete',$design_dir,'txt',$design);
}
elsif ($action eq 'DeleteDesign') {
	DeleteFile('Design',$design_dir);
}
elsif ($action eq 'Preview Design') {
	PreviewDesign();
}
elsif ($action eq 'Uploaded Images') {
	ManageFiles($upload_dir,"$pdurl/uploads",undef);
}
elsif ($action eq 'Cached Images') {
	ManageFiles($cache_dir,undef,undef);
}
elsif ($action eq 'DeleteImage') {
	my $image = $q->param('image');
	DeleteImage($image);
}
elsif ($action eq 'Stored Cards') {
	ManageFiles($postcard_dir,undef,'.card');
}
elsif ($action eq 'Usage Report') {
	UsageReportForm();
}
elsif ($action eq 'GenerateReport') {
	require 'pdlog.pm';
	GenReport($q->param('logfile'),$q->param('reporttype'));
}
elsif ($action eq 'Rotate Log File') {
	Operation('LogFile','Rotate',$log_dir,'log');
}
elsif ($action eq 'RotateLogFile') {
	RotateLog();
}
elsif ($action eq 'Delete Log File') {
	Operation('LogFile','Delete',$log_dir,'log');
}
elsif ($action eq 'DeleteLogFile') {
	DeleteFile('Log',$log_dir);
}
elsif ($action eq 'Change Password') {
	ChangePasswordForm();
}
elsif ($action eq 'ChangePassword') {
	ChangePassword();
}
elsif ($action eq 'Logout') {
	Logout();
}
else {
	Invalid($action);
}
	
#==============================================================================
# START OF SUBROUTINES.
#==============================================================================

#------------------------------------------------------------------------------
sub Login {
	my $cookie = $q->cookie(-name=>'cookietest',-value=>'test');

	print $q->header(-expires=>'+60s',-cookie=>$cookie);
	
	Error("Password file $admin_dir/auth is missing") unless (-f "$admin_dir/auth");

	# 
	# If the password is set to "password" or blank, then we need to set a 
	# real password.
	#
	open PASS,"$admin_dir/auth" or Error("Can not open $admin_dir/auth",$!);
	my ($user,$password) = split(/:/,<PASS>);
	chomp $password;
	close PASS;

	SetPasswordForm($user) if ($password =~ /password/i or $password eq '');

	$language = $lang;
	my $languages=ReadLanguages();

	Header("Postcard Direct Administration Login");

	print <<EOF;

<table border="0" width="50%" cellspacing="0" class="menutable">
<tr class="title">
 <td align="center" valign="center" colspan="2">
 Postcard Direct Administration Login
 </td>
</tr>

<form action="$script" method="post">
<tr>
 <td>Username:</td> <td><input type="text" name="user" size="20"></td>
</tr>

<tr>
<td>Password:</td> <td><input type="password" name="password" size="10"></td>
</tr>

<tr>
<td>Language:</td> <td><select name="language">$languages</select></td>
</tr>

<tr>
<td colspan=2>&nbsp;</td>
</tr>

<tr bgcolor="gray">
 <td>&nbsp;</td>
 <td><input type="submit" class="button" value="Login">
 <input type="reset" class="button" name="Reset">
 </td>
</tr>

</table>
<input type="hidden" name="action" value="CheckLogin">
</form>

<small>
<i>Ensure you have Javascript and cookies enabled for correct operation.</i>
<p>Version: $version</p>
</small>

</body>
</html>
EOF

	exit 0;

}

#------------------------------------------------------------------------------
sub CheckLogin {
	my $cookie = $q->cookie('cookietest');

 	unless ($cookie eq 'test') {
		print $q->header;
		Error("<h3>Looks like cookies are disabled!</h3>","Test cookie returned: $cookie");
	}

	my $user = $q->param('user');
	my $password = $q->param('password');

	($user,$password) = Encrypt($user,$password);

	my ($cookie1,$cookie2);
	
	if ($expire) {
		$cookie1 = $q->cookie(-name=>'user',-value=>$user,-expires=>$expire);
		$cookie2 = $q->cookie(-name=>'password',-value=>$password,-expires=>$expire);
	}
	else {
		$cookie1 = $q->cookie(-name=>'user',-value=>$user);
		$cookie2 = $q->cookie(-name=>'password',-value=>$password);
	}

	#
	# Old versions of the CGI module have a bug where the
	# cookie path is not set correctly. 
	#
	if ($CGI::VERSION < 2.68) {
		$cookie1 =~ s!path=\S+; !path=/; !;
		$cookie2 =~ s!path=\S+; !path=/; !;
	}

	print $q->header(-expires=>'+60s',-cookie=>[$cookie1,$cookie2]);

	if (CheckPass($user,$password)) {
		InvalidLogin();
	}
	else {
		MainMenu();
	}
}

#------------------------------------------------------------------------------
sub CheckAuth {
	my $user = $q->cookie('user');
	my $password = $q->cookie('password');

	unless ($user and $password) {
		print $q->header;
		Error("<h3>Looks like cookies are disabled!</h3>");
	}

	print $q->header(-expires=>'+60s');

	InvalidLogin() if (CheckPass($user,$password));
}

#------------------------------------------------------------------------------
sub MainMenu {
	my $hidden=Hidden();

	Header("Postcard Direct Administration",'javascript');

	print <<EOF;
<table border="0" cellspacing="1" cellpadding="2" class="menutable">
<tr class="title">
 <td align="center" valign="center" colspan="2">
 Postcard Direct Adminstration
 </td>
</tr>

<form action="$script" method="post">
EOF

	MenuSection("Language Options",$lang_name);
	MenuOption("Select Language","nr");
	MenuOption("Edit Messages",undef,"edit","$admin_dir/messages.txt");
	MenuOption("Edit Languages","nr","edit","$admin_dir/languages.txt");
	MenuOption(undef,"empty");

	MenuSection("Configuration Management",$configuration);
	MenuOption("Select Configuration","nr");
	MenuOption("Edit Configuration",undef,"edit","$config_dir/$configuration");
	MenuOption("Copy Configuration","nr");
	MenuOption("Delete Configuration");

	MenuSection("Template Management",$template);
	MenuOption("Select Template","nr");
	MenuOption("Edit Template",undef,"edit","$template_dir/$template");
	MenuOption("Copy Template","nr");
	MenuOption("Delete Template",);

	MenuSection("Design Management",$design);
	MenuOption("Select Design","nr");
	MenuOption("Edit Design",undef,"edit","$design_dir/$design");
	MenuOption("Copy Design","nr");
	MenuOption("Delete Design");
	MenuOption("Preview Design","nr","preview");
	MenuOption(undef,"empty");

	MenuSection("Stylesheet Management",$stylesheet);
	MenuOption("Select Stylesheet","nr");
	MenuOption("Edit Stylesheet",undef,"edit","$style_dir/$stylesheet");
	MenuOption("Copy Stylesheet","nr");
	MenuOption("Delete Stylesheet",);

	MenuSection("List File Management");
	MenuOption("Edit Bad User List","nr","edit","$admin_dir/badusers.txt");
	MenuOption("Edit remote sites List",undef,"edit","$admin_dir/remotesites.txt");
	MenuOption("Edit Domains List","nr","edit","$admin_dir/domains.txt");
	MenuOption("Edit MIME types",undef,"edit","$admin_dir/mimetypes.txt");
	MenuOption("Edit Bad Words List","nr","edit",$badwords_list);
	MenuOption("Edit Objects List",undef,"edit",$object_list);
	MenuOption("Edit Midi List","nr","edit",$midi_list);
	MenuOption("Edit Designs List",undef,"edit",$design_list);

	MenuSection("Log File Management");
	MenuOption("Usage Report","nr","usagereport");
	MenuOption("Rotate Log File");
	MenuOption("Delete Log File","nr");
	MenuOption(undef,"empty");

	MenuSection("Cache File Management");
	MenuOption("Uploaded Images","nr");
	MenuOption("Cached Images");
	MenuOption("Stored Cards","nr");
	MenuOption(undef,"empty");

	MenuSection("Account Management");
	MenuOption("Change Password","nr");
	MenuOption("Logout");

	print <<EOF;
$hidden
</form>
</table>

</body>
</html>
EOF

}

#------------------------------------------------------------------------------
sub SelectLanguage {
	my $languages=ReadLanguages();
	my $hidden=Hidden();

	Header("Language Selection");

	print <<EOF;
<table border="0" width="50%" cellspacing="0" class="menutable">
<tr class="title"
 <td align="center" valign="center" colspan="2">
 Postcard Direct Language Selection
 </td>
</tr>

<form action="$script" method="post">
<tr>
 <td>Language:</td> <td><select name="language">$languages</select></td>
</tr>

<tr>
<td colspan=2>&nbsp;</td>
</tr>

<tr bgcolor="gray">
 <td>&nbsp;</td>
 <td><input type="submit" value="Select">
 <input type="button" value="Cancel" class="button" onClick="history.go(-1);return true">
 </td>
</tr>

</table>
<input type="hidden" name="action" value="MainMenu">
$hidden
</form>

</div>

</body>
</html>
EOF

}

#------------------------------------------------------------------------------
sub MenuSection {
	my ($title,$subtitle)=@_;
	
	print <<EOF;
<tr class="menusection">
<td colspan="2" align="center">
$title
EOF

	print qq(<font size=-1>(<i><b>$subtitle</b></i>)</font>\n) if $subtitle;

}

#------------------------------------------------------------------------------
sub MenuOption {
	my ($title,$cell,$directive,$file)=@_;

	$cell=0 unless $cell;
	$directive=0 unless $directive;
	$file=0 unless $file;

	if ($cell eq 'nr') {
		print qq(<tr><td align="right">\n);
	}
	elsif ($cell eq 'empty') {
		print qq(<td>&nbsp;</td></tr>\n);
		return;
	}
	else {
		print qq(<td align="left">\n);
	}


	if ($directive eq 'edit') {
		print qq[<input type="button" class="menu" value="$title" onClick="edit('$pdedit?file=$file&action=Edit')">\n];
	}
	elsif ($directive eq 'preview') {
		print qq[<input type="button" class="menu" value="$title" onClick="newwindow('$script?action=Preview%20Design&language=$language&design=$design&configuration=$configuration')">\n];
	}
	elsif ($directive eq 'usagereport') {
		print qq[<input type="button" class="menu" value="$title" onClick="newwindow('$script?action=Usage%20Report&language=$language&design=$design&configuration=$configuration')">\n];
	}
	else {
		print qq[<input type="submit" name="action" class="menu" value="$title">\n];
	}

	print "</td>";
	print "</tr>\n" unless $cell;
}

#------------------------------------------------------------------------------
sub MenuSelect {
	my ($name,$list)=@_;

	print <<EOF;
<tr>
<td align="center"><select name="$name">
EOF
	foreach my $option (split(',',$list)) {
		print "<option>$option</option>\n";
	}

	print "</option></select></td></tr>\n";

}

#------------------------------------------------------------------------------
sub PreviewDesign {
	open(DESIGN,"$design_dir/$design") or Error("Can not open $design_dir/$design",$!);
	
	my $message = qq[This is a sample preview of what your postcard will look like using the <i>$design_dir/$design</i> postcard design.<p>If you have midi enabled, you should be hearing the soothing sounds of Vangelis.</p>];
	$message =~ s/\n/<br>/g;

	my $style=Stylesheet($stylesheet);

	while (<DESIGN>) {
		next if (/^#/);
		s!%stylesheet%|<pd_stylesheet>!$style!ig;
		s!%sender%|<pd_sender>!Postcard Direct!ig;
		s!%sender_email%|<pd_sender_email>!pd\@ginini.com!ig;
		s!%recipient%|<pd_recipient>!Joe Bloggs!ig;
		s!%recipient_email%|<pd_recipient_email>!joe\@bloggs.com!ig;
		s!%subject%|<pd_subject>!Preview of the Postcard!ig;
		s!%title%|<pd_title>!Postcard Preview!ig;
		s!%message%|<pd_message>!$message<p>It's really looking great.</p>!ig;
		s!%image%|<pd_image>!$images/photo.jpg!ig;
		s!%width%|<pd_width>!450!ig;
		s!%height%|<pd_height>!338!ig;
		s!%object%|<pd_object>!$images/sample.wav!ig;
		s!%midi%|<pd_midi>!<noembed><bgsound src="$pdurl/midi/antartica.mid" autostart="true"></bgsound></noembed> <embed src="$pdurl/midi/antartica.mid" autostart="true" hidden="true"></embed>!ig;
		s!%back%|<pd_back>!!ig;
		s!%send%|<pd_send>!!ig;
		print;
	}

	close(DESIGN);
}

#------------------------------------------------------------------------------
sub ManageFiles {
	my ($dir,$url,$ext) = @_;

	my $targeturl;
	my $sort = $q->param('sort') || 'File Name';
	my $order = $q->param('order') || 'Ascending';

	my @sorttype=('File Name','File Age','File Size');
	my @ordertype=('Ascending','Descending');

	Header("$action",1);

	print qq[<h2>$action</h2>\n];

	print <<EOF;

<form action=$script method="post">
<select name="sort">
EOF


foreach my $option (@sorttype) {
	if ($option eq $sort) {
		print "<option selected>$option</option>\n";
	}
	else {
		print "<option>$option</option>\n";
	}
}

print <<EOF;
</select>

<select name="order">
EOF

foreach my $option (@ordertype) {
	if ($option eq $order) {
		print "<option selected>$option</option>\n";
	}
	else {
		print "<option>$option</option>\n";
	}
}


print <<EOF;
</select>

<input type="hidden" name="action" value="$action">
<input type="submit" value="Sort">
</form>
<br>


<table border="0" cellpadding="0" cellspacing="15" class="filelist">
EOF

	my %fileorder;

	foreach my $file (<$dir/*$ext>) {
		next unless -f $file;

		if ($sort eq 'File Age') {
			$fileorder{$file} = -M $file;
		}
		elsif ($sort eq 'File Size') {
			$fileorder{$file} = -s $file;
		}
		elsif ($sort eq 'File Name') {
			$fileorder{$file} = $file;
		}
		else {
			Error("$sort is an invalid sort type");
		}
			
	}

	my @sorted;

	if ($sort eq 'File Name') {
		if ($order eq 'Ascending') {
			@sorted = sort { $fileorder{$a} cmp $fileorder{$b} } keys %fileorder;
		}
		else {
			@sorted = sort { $fileorder{$b} cmp $fileorder{$a} } keys %fileorder;
		}
	}
	elsif ($sort eq 'File Age') {
		if ($order eq 'Ascending') {
			@sorted = sort { $fileorder{$b} <=> $fileorder{$a} } keys %fileorder;
		}
		else {
			@sorted = sort { $fileorder{$a} <=> $fileorder{$b} } keys %fileorder;
		}
	}
	else {
		if ($order eq 'Ascending') {
			@sorted = sort { $fileorder{$a} <=> $fileorder{$b} } keys %fileorder;
		}
		else {
			@sorted = sort { $fileorder{$b} <=> $fileorder{$a} } keys %fileorder;
		}
	}

	foreach my $file (@sorted) {

		my ($size,$age) = (stat($file))[7,9];
		$age=localtime($age);
		$size = int($size / 1024);

		my $bfile = basename($file);

		if ($url) {
			$targeturl="$url/$bfile";
		}
		else {
			$targeturl="$pdview?file=$file";
		}
		print qq[<tr><td>$bfile</td><td>$size KB</td><td>$age</td>];
		print qq[<td><input type="button" value="View" class="button" onClick="edit('$targeturl')"></td>];
		print qq[<form><td><input type="submit" value="Delete" class="button"><input type="hidden" name="action" value="DeleteImage"></td><input type="hidden" name="image" value="$file"><input type="hidden" name="actiontype" value="$action"></form></tr>\n];
	}

	print "</table><p></p>\n";

	MenuReturn();

}

#------------------------------------------------------------------------------
sub DeleteImage {
	my $file = shift;

	Error("$file does not exist") unless -f $file;
	unlink $file or Error("Can not delete $file",$!);

	print "<p>$file successfully deleted</p>\n";

	my $action = $q->param('actiontype');

	my $hidden=Hidden();

	print <<EOF;
<form action="$script" method="post">
$hidden
<input type="hidden" name="action" value="$action">
<input type="submit" value="Continue">
</form>
EOF

}

#------------------------------------------------------------------------------
sub UsageReportForm {
	my $options;

	foreach my $option (SelectFiles($log_dir,"log")) {
		if ($option eq $log_file) {
			$options .= "<option selected>$option</option>";
		}
		else {
			$options .= "<option>$option</option>";
		}
	}

	unless ($options) {
		Header("Usage Report",undef,"left");
		print "<h2>No log files found</h2>\n";
 		print qq(<form><input type="button" value="Close Window" class="button" onClick="window.close()"></form>\n);
		Footer();
		return;
	}

	my $hidden=Hidden();

	Header("Usage Report");

	print <<EOF;
<table border="0" width="50%" cellspacing="0" class="menutable">
<tr class="title"
 <td align="center" valign="center" colspan="2">
 Usage Report
 </td>
</tr>

<form action="$script" method="post">
<tr>
 <td>Log File:</td> <td><select name="logfile">$options</select></td>
</tr>

<tr>
 <td>Report Type:</td> <td><input type="radio" name="reporttype" value="summary" checked>Summary<br><input type="radio" name="reporttype" value="detailed">Detailed</td>
</tr>

<tr>
<td colspan="2">&nbsp;</td>
</tr>

<tr bgcolor="gray">
 <td>&nbsp;</td>
 <td><input type="submit" class="button" value="Generate Report">
 <input type="button" value="Cancel" class="button" onClick="window.close()">
 </td>
</tr>

</table>
<input type="hidden" name="action" value="GenerateReport">
$hidden
</form>

</div>

</body>
</html>
EOF

}

#------------------------------------------------------------------------------
sub SetPasswordForm {
	my $user = shift;

	Header("Set Initial Username/Password");

	print <<EOF;
<table border="0" width="50%" cellspacing="0" class="menutable">
<tr class="title">
 <td align="center" valign="center" colspan="2">
 Set Initial Username/Password
 </td>
</tr>

<form action="$script" method="post">
<tr>
 <td>Username:</td> <td><input type="text" name="user" size="20" value="$user"></td>
</tr>

<tr>
 <td>Password:</td> <td><input type="password" name="password" size="10"></td>
</tr>

<tr>
 <td>Confirm New Password:</td> <td><input type="password" name="password1" size="10"></td>
</tr>

<tr>
<td colspan=2>&nbsp;</td>
</tr>

<tr bgcolor="gray">
 <td>&nbsp;</td>
 <td><input type="submit" class="button" value="Create Account">
 <input type="reset" class="button" name="Reset">
 </td>
</tr>

</table>
<input type="hidden" name="action" value="SetPassword">
</form>

</div>

</body>
</html>
EOF

	exit;
}

#------------------------------------------------------------------------------
sub SetPassword {
	my $user=$q->param('user');
	my $password=$q->param('password');
	my $password1=$q->param('password1');

	#
	# Check to make sure the password really is the default before
	# going and setting it.
	#
	open PASS,"$admin_dir/auth" or Error("Can not open $admin_dir/auth",$!);
	my ($cuser,$cpassword) = split(/:/,<PASS>);
	chomp $cpassword;
	close PASS;

	unless ($cpassword =~ /password/i or $cpassword eq '') {
		print $q->header(-expires=>'+60s');
		Error("Can not set password.  Password already exists");
	}

	PasswordErr("Password can not be blank") unless ($password or $password1);
	PasswordErr("New passwords don't match") if ($password ne $password1);

	chomp $password;

	my ($duser,$dpassword) = Encrypt($user,$password);

	my ($cookie1,$cookie2);

	if ($expire) {
		$cookie1 = $q->cookie(-name=>'user',-value=>$duser,-expires=>$expire);
		$cookie2 = $q->cookie(-name=>'password',-value=>$dpassword,-expires=>$expire);
	}
	else {
		$cookie1 = $q->cookie(-name=>'user',-value=>$duser);
		$cookie2 = $q->cookie(-name=>'password',-value=>$dpassword);
	}

	#
	# Old versions of the CGI module have a bug where the
	# cookie path is not set correctly. 
	#

	if ($CGI::VERSION < 2.68) {
		$cookie1 =~ s!path=\S+; !path=/; !;
		$cookie2 =~ s!path=\S+; !path=/; !;
	}

	print $q->header(-expires=>'+60s',-cookie=>[$cookie1,$cookie2]);

	my $apassword = encrypt($password,$tkey);

	my $authfile="$admin_dir/auth";

	open(AUTH,">$authfile") or Error("Can not write to $authfile",$!);
	print AUTH "$user:$apassword\n";
	close(AUTH);

	Header("Password Set",undef,'left');

	print <<EOF;
<h3>Password successfully set</h3>

<form action="$script" method="post">
<input type="submit" class="button" value="Continue">
</form>
EOF
	Footer();

	exit;
}

#------------------------------------------------------------------------------
sub ChangePasswordForm {
	my $hidden=Hidden();

	Header("Change Password");

	print <<EOF;
<table border="0" width="50%" cellspacing="0" class="menutable">
<tr class="title">
 <td align="center" valign="center" colspan="2">
 Change Password
 </td>
</tr>

<form action="$script" method="post">
<tr>
 <td>Username:</td> <td><input type="text" name="user" size="20"></td>
</tr>

<tr>
 <td>Old Password:</td> <td><input type="password" name="password" size="10"></td>
</tr>

<tr>
 <td>New Password:</td> <td><input type="password" name="password1" size="10"></td>
</tr>

<tr>
 <td>Confirm New Password:</td> <td><input type="password" name="password2" size="10"></td>
</tr>

<tr>
<td colspan=2>&nbsp;</td>
</tr>

<tr bgcolor="gray">
 <td>&nbsp;</td>
 <td><input type="submit" class="button" value="Change Password">
 <input type="reset" class="button" name="Reset">
 </td>
</tr>

</table>
<input type="hidden" name="action" value="ChangePassword">
$hidden
</form>

</div>

</body>
</html>
EOF

}

#------------------------------------------------------------------------------
sub ChangePassword {
	my $user = $q->cookie('user');
	my $password = $q->cookie('password');

	my $puser=$q->param('user');
	my $opassword=$q->param('password');
	my $npassword1=$q->param('password1');
	my $npassword2=$q->param('password2');

	PasswordErr("Password can not be blank") unless ($npassword1 or $npassword2);
	PasswordErr("New passwords don't match") if ($npassword1 ne $npassword2);
	my ($duser,$dpassword) = Encrypt($puser,$opassword);

	PasswordErr("Old Username or password does not match") if (CheckPass($duser,$dpassword));

	chomp $npassword1;
	my ($foo,$dnpassword) = Encrypt($puser,$npassword1);

	my ($cookie1,$cookie2);
	
	if ($expire) {
		$cookie1 = $q->cookie(-name=>'user',-value=>$duser,-expires=>$expire);
		$cookie1 = $q->cookie(-name=>'user',-value=>$duser,-expires=>$expire);
	}
	else {
		$cookie2 = $q->cookie(-name=>'password',-value=>$dnpassword);
		$cookie2 = $q->cookie(-name=>'password',-value=>$dnpassword);
	}

	#
	# Old versions of the CGI module have a bug where the
	# cookie path is not set correctly. 
	#

	if ($CGI::VERSION < 2.68) {
		$cookie1 =~ s!path=\S+; !path=/; !;
		$cookie2 =~ s!path=\S+; !path=/; !;
	}

	print $q->header(-expires=>'+60s',-cookie=>[$cookie1,$cookie2]);

	my $authfile="$admin_dir/auth";

	open(AUTH,">$authfile") or Error("Can not write to $authfile",$!);
	print AUTH "$puser:" . encrypt($npassword1,$tkey) . "\n";
	close(AUTH);

	Header("Password Changed",undef,'left');
	print "<h3>Password successfully changed</h3>\n";
	MenuReturn();
	Footer();
}

#------------------------------------------------------------------------------
sub PasswordErr {
	my $error=shift;

	print $q->header(-expires=>'+60s');
	Header("Password Change Error",undef,'left');
	print qq(<h1>Password Change Error</h1>\n);
	print qq(<div class="error">$error</div>\n);
	MenuReturn();
	Return2Form();
	Footer();
	exit;
}

#------------------------------------------------------------------------------
#sub VerifyConfiguration {
#	my $errfile='errfile';
#	open(STDERR,">$errfile") or Error("Can not redirect STDERR to $errfile",$!);
#
#	system("$^X -c $configfile");
#	my $hidden=Hidden();
#
#	open(ERR,$errfile) or Error("Can not open $errfile",$!);
#	my @results=<ERR>;
#	close(ERR);
#
#	Header("Configuration Verification");
#
#	print <<EOF;
#<h1>Configuration Verification</h1>
#<h3>Configuration: $configuration</h3>
#
#<code>
#@results
#</code>
#EOF
#
#	MenuReturn();
#}

#------------------------------------------------------------------------------
sub Logout {
	my $hidden=Hidden();

	my $cookie1 = $q->cookie(-name=>'user',-value=>'xxx');
	my $cookie2 = $q->cookie(-name=>'password',-value=>'xxx');

	#
	# Old versions of the CGI module have a bug where the
	# cookie path is not set correctly. 
	#

	if ($CGI::VERSION < 2.68) {
		$cookie1 =~ s!path=\S+!path=/!;
		$cookie2 =~ s!path=\S+!path=/!;
	}

	print $q->header(-expires=>'now',-cookie=>[$cookie1,$cookie2]);

	Header("Logged out",undef,"left");

	print <<EOF;
<h2>Logged out</h2>
Click <a href="$script?action=Login">here</a> to log back in.
EOF

	Footer();
}

#------------------------------------------------------------------------------
sub Invalid {
	my $action=shift;

	Header("Invalid Option",undef,"left");
	print qq(<h2>Invalid Option - <i>$action</i></h2>\n);
	MenuReturn();
	Footer();	
}

#------------------------------------------------------------------------------
sub Hidden {
	$language='en|English' unless $language;
	$configuration='pdconfig.txt' unless $configuration;
	$design='default.txt' unless $design;
	$template='form-default.txt' unless $template;

	return <<EOF;
<input type="hidden" name="language" value="$language"> 
<input type="hidden" name="configuration" value="$configuration"> 
<input type="hidden" name="design" value="$design"> 
<input type="hidden" name="stylesheet" value="$stylesheet"> 
<input type="hidden" name="template" value="$template"> 
EOF
}

#------------------------------------------------------------------------------
sub Javascript {

	return <<EOF;
<script language="javascript">
<!-- Begin
function edit(file)
{
  var url=file;
  var w=700,h=500;

  leftpos = (screen.width) ? (screen.width-w)/2 : 0;
  toppos = (screen.height) ? (screen.height-h)/2 : 0;
  
  window.open(url,'edit','toolbar=no,location=no,directories=no,menubar=no,resizable=yes,copyhistory=no,scrollbars=yes,width='+w+',height='+h+',top='+toppos+',left='+leftpos+',status=no');
}

function newwindow(file)
{
  var url=file
  window.open(url,'window','toolbar=yes,location=no,directories=no,status=no,menubar=no,resizable=yes,copyhistory=no,scrollbars=yes');
}
//-->
</script>
EOF
}

#------------------------------------------------------------------------------
sub Header {
	my ($title,$javascript,$align)=@_;

	my $style=Stylesheet('pdadmin.css');

	if ($javascript) {
		$javascript=Javascript();
	}
	else {
		$javascript='';
	}

	$align='center' unless $align;

	print <<EOF;
<html>
<head>
<title>$title</title>
$style
$javascript
</head>

<body bgcolor="#ffffff">
<div align="$align">
EOF
}

#------------------------------------------------------------------------------
sub Footer {
	print <<HTML;
</div>
<br>
<hr>
<small><i>pdadmin version: $version</i></small>
</body>
</html>
HTML
}


#------------------------------------------------------------------------------
sub SelectFiles {
	my ($dir,$pattern)=@_;

	my @files;

	Error("Directory $dir does not exist") unless -d $dir;

	$pattern='*' unless $pattern;

	foreach my $file (glob("$dir/*.$pattern")) {
		next if ($file =~ /\.bak/);
		push @files,basename($file);
	}

	return @files;
}

#------------------------------------------------------------------------------
sub ReadLanguages {
	my $languages;

	open(LANG,"$admin_dir/languages.txt") or Error("Can not open $admin_dir/languages.txt",$!);

	while (<LANG>) {
		next if (/^#/ or /^\s*$/);
		chomp;
		my ($code,$description) = split(/\|/);
		$description =~ s/\W+//g;

		if ($language eq $_ or $language eq $code) {
			$languages .= qq(<option value="$code|$description" selected>$description</option>\n);
		}
		else {
			$languages .= qq(<option value="$code|$description">$description</option>\n);
		}
	}

	close(LANG);

	return $languages;
}

#------------------------------------------------------------------------------
sub Operation {
	my ($name,$type,$dir,$pattern,$select)=@_;

	my $hidden=Hidden();

	my $options;

	foreach my $option (SelectFiles($dir,"$pattern")) {
		if ($option eq $select) {
			$options .= "<option selected>$option</option>";
		}
		else {
			$options .= "<option>$option</option>";
		}
	}


	unless ($options) {
		Header("$type $name",undef,"left");
		print "<h2>No files found</h2>\n";
		MenuReturn();
		Footer();
		return;
	}

	Header("$type $name");

	my ($menu,$selectname);

	if ($type eq 'Select') {
		$menu="MainMenu";
		$selectname = $name; 
	}
	else {
		$menu="$type$name";
		$selectname = "$type$name"; 
	}
	
	$selectname =~ tr/A-Z/a-z/;

	print <<EOF;
<table border=0 width=50% cellspacing=0 class="menutable">
<tr class="title">
 <td align=center valign=center colspan=2>
 $type $name
 </td>
</tr>

<form action="$script" method="post">
<tr>
EOF

	if ($type eq 'Copy') {
		print qq(<td>$name file to copy from:</td> <td><select name="file">$options</select></td>\n);
		print qq(</tr>\n<tr>\n);
		print qq(<tr><td>$name file name to copy to:</td>\n);
 		print qq(<td><input type="text" name="file1"></td>\n);
	}
	elsif ($type eq 'Rotate') {
		print qq(<td>$name:</td> <td><select name="file">$options</select></td>\n);
		print qq(</tr>\n<tr>\n);
		print qq(<tr><td>New name:</td>\n);
 		print qq(<td><input type="text" name="file1"></td>\n);
	}
	elsif ($type eq 'Delete') {
		print qq(<tr>\n<td>$name:</td> <td><select name="file">$options</select></td>\n</tr>\n);
	}
	else {
		print qq(<tr>\n<td>$name:</td> <td><select name="$selectname">$options</select></td>\n</tr>\n);
	}

	print <<EOF;
</tr>

<tr>
<td colspan="2">&nbsp;</td>
</tr>

<tr bgcolor="gray">
 <td>&nbsp;</td>
 <td><input type="submit" class="button" value="$type">
 <input type="button" value="Cancel" class="button" onClick="history.go(-1);return true">
 </td>
</tr>

</table>
<input type="hidden" name="action" value="$menu">
$hidden
</form>

</div>

</body>
</html>
EOF

}

#------------------------------------------------------------------------------
sub CopyFile {
	my ($type,$dir) = @_;

	my $file=$q->param('file');
	my $file1=$q->param('file1');

	InputError("$type file name to copy from not specified") unless $file;
	InputError("$type file name to copy to, not specified") unless $file1;

	InputError("Illegal path: $file") if CheckBadPath($file,undef);
	InputError("Illegal path: $file1") if CheckBadPath($file1,undef);

	Error("$type $file does not exist") unless (-f "$dir/$file");
	Error("$type $file1 already exists") if (-f "$dir/$file1");

	copy("$dir/$file","$dir/$file1") or Error("Could not copy $file to $file1",$!);

	Results("Copied $type File","From: $file<br>To: $file1");
}

#------------------------------------------------------------------------------
sub DeleteFile {
	my ($type,$dir) = @_;

	my $file=$q->param('file');

	InputError("$type file not specified") unless $file;
	InputError("Illegal path: $file") if CheckBadPath($file,undef);

	Error("$type $file does not exist") unless (-f "$dir/$file");
	unlink "$dir/$file" or Error("Can not delete $type $file",$!);

	if ($type eq 'Configuration') {
		$configuration='pdconfig.txt';
	}
	elsif ($type eq 'Template') {
		$template='form-default.txt';
	}
	elsif ($type eq 'Design') {
		$design='default.txt';
	}

	Results("Deleted $type","Sucessfully deleted $type file: $file");
}

#------------------------------------------------------------------------------
sub RotateLog {
	my $logfile=$q->param('file');
	my $newlogfile=$q->param('file1');

	InputError("Log file not specified") unless $logfile;
	InputError("New log file not specified") unless $newlogfile;
	InputError("Illegal path: $logfile") if CheckBadPath($logfile,undef);
	InputError("Illegal path: $newlogfile") if CheckBadPath($newlogfile,undef);

	Error("Log file $newlogfile already exists") if (-f "$log_dir/$newlogfile");

	move("$log_dir/$logfile","$log_dir/$newlogfile") or Error("Can not move log file $logfile to $newlogfile",$!);

	Results("Rotate Log File","Sucessfully rotated log file $logfile to $newlogfile");
}

#------------------------------------------------------------------------------
sub MenuReturn {
	my $hidden=Hidden();

	print <<EOF;
<form action="$script" method="post">
$hidden
<input type="hidden" name="action" value="MainMenu">
<input type="submit" value="Return to Main Menu">
</form>
EOF
}

#------------------------------------------------------------------------------
sub InvalidLogin {
	Header("Invalid Login",undef,"left");
	print qq(<h2>Invalid login</h2>\n);
	print qq(You have either entered an incorrect username or password, your session has expired, or your webserver time has drifted too far.<br>);
	print qq(<p>Return to <a href="$script">Login page</a></p>\n);
	Footer();
	exit;
}

#------------------------------------------------------------------------------
sub InputError {
	my $message=shift;

	Header($message,undef,"left");
	print qq(<h2>Input Error</h2>\n);
	print "$message\n";
	Return2Form();
	Footer();
	exit;
}

#------------------------------------------------------------------------------
sub Return2Form {
	print <<HTML;
<form>
<input type="button" value="Return to form" class="button" onClick="history.go(-1);return true">
</form>
HTML
}

#------------------------------------------------------------------------------
sub Results {
	my ($title,$message)=@_;

	Header($title,undef,"left");
	print qq(<h3>$title</h3>\n);
	print "$message\n";
	MenuReturn();	
	Footer();
}
